/* image.c - handle the client copy of the image
 *
 * Copyright (C) 1999  Jochen Voss.  */

static const  char  rcsid[] = "$Id: image.c 5044 2003-07-09 08:02:23Z voss $";

#include <stdlib.h>
#include <assert.h>

#include <X11/Xutil.h>

#include "libv/v.h"
#include "xising.h"


struct image *
new_image (Window win, int width, int height)
{
  struct image *res;
  int  llen;
  char *buffer;

  res = xmalloc (sizeof (struct image));
  res->win = win;
  res->width = width;
  res->height = height;
  res->x_off = 0;
  res->y_off = 0;
  llen = (width+7)/8;
  buffer = calloc (llen*height, 1);
  assert (buffer != 0);
  res->data = XCreateImage (display, DefaultVisual (display, screen_num),
			1, XYBitmap, 0, buffer,
			width, height,
			8, llen);
  assert (res->data != 0);
  res->energy = 2*(width-1)*height+2*width*(height-1);
  res->magn = -width*height;
  
  return  res;
}

void
delete_image (struct image *image)
{
  XDestroyImage (image->data);
  xfree (image);
}

void
image_redraw (const struct image *im, int x, int y, int w, int h)
{
  XPutImage (display, im->win, gc, im->data,
	     x, y, x+im->x_off, y+im->y_off, w, h);
}

int
image_energy (const struct image *im)
/* Calculate the absolute energy of the image */
{
  int  h, i, j;

  h = 0;
  for (i=0; i<im->width-1; ++i) {
    for (j=0; j<im->height; ++j) {
      h += XGetPixel(im->data, i, j) != XGetPixel(im->data, i+1, j) ? +2 : -2;
    }
  }
  for (i=0; i<im->width; ++i) {
    for (j=0; j<im->height-1; ++j) {
      h += XGetPixel(im->data, i, j) != XGetPixel(im->data, i, j+1) ? +2 : -2;
    }
  }
  
  return  h;
}

int
image_magnetisation (const struct image *im)
/* Calculate the absolute magnetisation of the image */
{
  int  b = 0;
  int  i, j;

  for (i=0; i<im->width; ++i) {
    for (j=0; j<im->height; ++j) {
      if (XGetPixel (im->data, i, j)) {
	b += 1;
      } else {
	b -= 1;
      }
    }
  }
  
  return  b;
}

void
image_fill_random (struct image *im)
{
  int  x, y;

  for (x=0; x<im->width; ++x) {
    for (y=0; y<im->height; ++y) {
      int  val;
#if HAVE_DRAND48
      val = (drand48()<0.5) ? 0 : 1;
#else
      val = (rand()<RAND_MAX/2) ? 0 : 1;
#endif
      XPutPixel (im->data, x, y, val);
    }
  }
  im->energy = image_energy (im);
  im->magn = image_magnetisation (im);
}

void
image_fill_white (struct image *im)
{
  int  w, h;
  int  x, y;

  w = im->width;
  h = im->height;
  for (x=0; x<w; ++x) {
    for (y=0; y<h; ++y)  XPutPixel (im->data, x, y, 0);
  }
  im->energy = 2*(w-1)*h+2*w*(h-1);
  im->magn = -w*h;
}

void
image_fill_black (struct image *im)
{
  int  w, h;
  int  x, y;

  w = im->width;
  h = im->height;
  for (x=0; x<w; ++x) {
    for (y=0; y<h; ++y)  XPutPixel (im->data, x, y, 1);
  }
  im->energy = 2*(w-1)*h+2*w*(h-1);
  im->magn = w*h;
}

int
image_equal (const struct image *im1, const struct image *im2)
{
  int  x, y;
  
  if (im1->energy != im2->energy)  return 0;
  if (im1->magn != im2->magn)  return 0;
  if (im1->width != im2->width || im1->height != im2->height)  return 0;

  for (x=0; x<im1->width; ++x) {
    for (y=0; y<im1->height; ++y) {
      if (XGetPixel(im1->data, x, y) != XGetPixel(im2->data, x, y))  return 0;
    }
  }
  return 1;
}
